@@ -35,11 +35,10 @@ from typing import Optional
 from typing import Tuple
 
 from openshift.dynamic.exceptions import NotFoundError as OpenShiftNotFoundError
-from .exceptions import ThothCommonException
-from .exceptions import NotKnownThothIntegration
-from .exceptions import QebHwtInputsMissing
-from .exceptions import KebechetInputsMissing
-from .exceptions import NotFoundException
+from .exceptions import ThothCommonExceptionError
+from .exceptions import NotKnownThothIntegrationError
+from .exceptions import KebechetInputsMissingError
+from .exceptions import NotFoundExceptionError
 from .exceptions import ConfigurationError
 from .exceptions import SolverNameParseError
 from .helpers import (
@@ -47,8 +46,9 @@ from .helpers import (
     _get_incluster_token_file,
     get_service_account_token,
     normalize_os_version,
+    format_datetime,
 )
-from .enums import ThothAdviserIntegrationEnum
+from .enums import ThothAdviserIntegrationEnum, InternalTriggerEnum
 
 from typing import TYPE_CHECKING
 
@@ -71,6 +71,7 @@ class OpenShift:
         middletier_namespace: Optional[str] = None,
         backend_namespace: Optional[str] = None,
         infra_namespace: Optional[str] = None,
+        graph_namespace: Optional[str] = None,
         amun_infra_namespace: Optional[str] = None,
         amun_inspection_namespace: Optional[str] = None,
         kubernetes_api_url: Optional[str] = None,
@@ -147,6 +148,7 @@ class OpenShift:
         self.backend_namespace = backend_namespace or os.getenv(
             "THOTH_BACKEND_NAMESPACE"
         )
+        self.graph_namespace = graph_namespace or os.getenv("THOTH_GRAPH_NAMESPACE")
         self.infra_namespace = infra_namespace or os.getenv("THOTH_INFRA_NAMESPACE")
         self.kubernetes_api_url = kubernetes_api_url or os.getenv(
             "KUBERNETES_API_URL", "https://kubernetes.default.svc.cluster.local"
@@ -345,7 +347,7 @@ class OpenShift:
         )
 
         if response.status_code == 404:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"Pod with id {pod_id} was not found in namespace {namespace}"
             )
 
@@ -357,7 +359,7 @@ class OpenShift:
             # If Pod has not been initialized yet, there is returned 400 status code. Return None in this case.
             return None
         elif response.status_code == 400:
-            raise ThothCommonException(
+            raise ThothCommonExceptionError(
                 f"Failed to obtain logs, container name has to be specified: {response.json()['message']}"
             )
 
@@ -365,7 +367,7 @@ class OpenShift:
             response.raise_for_status()
         except Exception as exc:
             _LOGGER.error("Error response when obtaining pod logs: %r", response.json())
-            raise ThothCommonException(
+            raise ThothCommonExceptionError(
                 f"Failed to obtain logs for pod: {str(exc)}"
             ) from exc
 
@@ -382,7 +384,7 @@ class OpenShift:
             if node_info["displayName"] == node_name:
                 return pod_name  # type: ignore
 
-        raise NotFoundException(
+        raise NotFoundExceptionError(
             f"No node named {node_name!r} found in workflow {workflow_id!r} in namespace {namespace!r}"
         )
 
@@ -417,7 +419,7 @@ class OpenShift:
         )
 
         if response.status_code == 404:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"Build with id {build_id} was not found in namespace {namespace}"
             )
 
@@ -448,7 +450,7 @@ class OpenShift:
         )
 
         if response.status_code == 404:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"BuildConfig with id {buildconfig_id} was not found in namespace {namespace}"
             )
 
@@ -479,7 +481,7 @@ class OpenShift:
         )
 
         if response.status_code == 404:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"Build with id {build_id} was not found in namespace {namespace}"
             )
 
@@ -501,7 +503,7 @@ class OpenShift:
                 namespace=namespace, name=pod_id
             )
         except openshift.dynamic.exceptions.NotFoundError as exc:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"The given pod with id {pod_id} could not be found"
             ) from exc
 
@@ -583,7 +585,9 @@ class OpenShift:
                     f"Multiple pods for the same job name selector {job_id} found"
                 )
 
-            raise NotFoundException(f"Job with the given id {job_id} was not found")
+            raise NotFoundExceptionError(
+                f"Job with the given id {job_id} was not found"
+            )
 
         result: str = response["items"][0]["metadata"]["name"]
 
@@ -604,7 +608,9 @@ class OpenShift:
         _LOGGER.debug("OpenShift response for pod ids from job: %r", response)
 
         if not len(response.get("items", [])):
-            raise NotFoundException(f"Job with the given id {job_id} was not found")
+            raise NotFoundExceptionError(
+                f"Job with the given id {job_id} was not found"
+            )
 
         result: List[str] = [pod["metadata"]["name"] for pod in response["items"]]
 
@@ -619,7 +625,7 @@ class OpenShift:
             )
             return result
         except OpenShiftNotFoundError as exc:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"Configmap {configmap_id!r} not found in namespace {namespace!r}"
             ) from exc
 
@@ -670,7 +676,7 @@ class OpenShift:
             job_status["completions"] = job["spec"]["completions"]
 
         except OpenShiftNotFoundError as exc:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"Job {job_id!r} not found in namespace {namespace!r}"
             ) from exc
 
@@ -711,7 +717,7 @@ class OpenShift:
                 api_version="batch/v1", kind="Job"
             ).get(namespace=namespace, label_selector=label_selector)
         except openshift.dynamic.exceptions.NotFoundError as exc:
-            raise NotFoundException(
+            raise NotFoundExceptionError(
                 f"No Jobs with label {label_selector} could be found"
             ) from exc
 
@@ -886,6 +892,21 @@ class OpenShift:
             },
         )
 
+    def get_cache_expiration_configuration(self) -> Optional[int]:
+        """Retrieve cache expiration configuration [s]."""
+        if not self.frontend_namespace:
+            raise ConfigurationError(
+                "Frontend namespace is required in order to list solvers"
+            )
+
+        cm = self.get_configmap("user-api", namespace=self.frontend_namespace)
+        cache_expiration_time: Optional[int] = cm["data"].get("cache-expiration")
+        if not cache_expiration_time:
+            _LOGGER.warning("No cache expiration time identifed in user-api ConfigMap")
+            return None
+
+        return cache_expiration_time
+
     def get_solver_names(self) -> List[str]:
         """Retrieve name of solvers available in installation."""
         if not self.infra_namespace:
@@ -905,6 +926,7 @@ class OpenShift:
         packages: str,
         *,
         indexes: Optional[List[str]] = None,
+        force_sync: bool = False,
         debug: bool = False,
         transitive: bool = False,
     ) -> typing.List[str]:
@@ -915,6 +937,7 @@ class OpenShift:
                 packages=packages,
                 solver=solver_name,
                 indexes=indexes,
+                force_sync=force_sync,
                 debug=debug,
                 transitive=transitive,
             )
@@ -929,6 +952,7 @@ class OpenShift:
         solver: str,
         *,
         indexes: Optional[List[str]] = None,
+        force_sync: bool = False,
         debug: bool = False,
         transitive: bool = True,
         job_id: Optional[str] = None,
@@ -946,6 +970,7 @@ class OpenShift:
             "THOTH_SOLVER_PACKAGES": packages.replace("\n", "\\n"),
             "THOTH_SOLVER_NO_TRANSITIVE": int(not transitive),
             "THOTH_SOLVER_INDEXES": ",".join(indexes) if indexes else "",
+            "THOTH_FORCE_SYNC": int(force_sync),
             "THOTH_LOG_SOLVER": "DEBUG" if debug else "INFO",
         }
 
@@ -1267,26 +1292,11 @@ class OpenShift:
             source_type not in ThothAdviserIntegrationEnum.__members__
             and source_type is not None
         ):
-            raise NotKnownThothIntegration(
+            raise NotKnownThothIntegrationError(
                 f"This integration {source_type} is not provided \
                     in Thoth: {ThothAdviserIntegrationEnum.__members__.keys()}"
             )
 
-    @staticmethod
-    def verify_github_app_inputs(
-        github_event_type: Optional[str],
-        github_check_run_id: Optional[int],
-        github_installation_id: Optional[int],
-        github_base_repo_url: Optional[str],
-        origin: Optional[str],
-    ) -> None:
-        """Verify if Thoth GitHub App integration inputs are correct."""
-        parameters = locals()
-        if not all(parameters.values()):
-            raise QebHwtInputsMissing(
-                f"Not all inputs to schedule Qeb-Hwt GitHub App are provided: {parameters}"
-            )
-
     @staticmethod
     def verify_kebechet_inputs(
         origin: Optional[str],
@@ -1294,29 +1304,16 @@ class OpenShift:
         """Verify if Thoth Kebechet integration inputs are correct."""
         parameters = locals()
         if not all(parameters.values()):
-            raise KebechetInputsMissing(
+            raise KebechetInputsMissingError(
                 f"Not all inputs to schedule Kebechet are provided: {parameters}"
             )
 
     def verify_integration_inputs(
         self,
         source_type: Optional[ThothAdviserIntegrationEnum],
-        github_event_type: Optional[str] = None,
-        github_check_run_id: Optional[int] = None,
-        github_installation_id: Optional[int] = None,
-        github_base_repo_url: Optional[str] = None,
         origin: Optional[str] = None,
     ) -> None:
         """Verify if inputs for registered Thoth integrations are correct."""
-        if source_type is ThothAdviserIntegrationEnum.GITHUB_APP:
-            self.verify_github_app_inputs(
-                github_event_type=github_event_type,
-                github_check_run_id=github_check_run_id,
-                github_installation_id=github_installation_id,
-                github_base_repo_url=github_base_repo_url,
-                origin=origin,
-            )
-
         if source_type is ThothAdviserIntegrationEnum.KEBECHET:
             self.verify_kebechet_inputs(origin=origin)
 
@@ -1329,14 +1326,14 @@ class OpenShift:
         predictor_config: Optional[Dict[str, Any]] = None,
         origin: Optional[str] = None,
         dev: bool = False,
+        authenticated: bool = False,
         debug: bool = False,
         job_id: Optional[str] = None,
-        github_event_type: Optional[str] = None,
-        github_check_run_id: Optional[int] = None,
-        github_installation_id: Optional[int] = None,
-        github_base_repo_url: Optional[str] = None,
         re_run_adviser_id: Optional[str] = None,
         source_type: Optional[str] = None,
+        kebechet_metadata: Optional[Dict[str, Any]] = None,
+        justification: Optional[List[Dict[str, Any]]] = None,
+        stack_info: Optional[List[Dict[str, Any]]] = None,
     ) -> Optional[str]:
         """Schedule an adviser run."""
         if not self.backend_namespace:
@@ -1352,10 +1349,6 @@ class OpenShift:
 
         self.verify_integration_inputs(
             source_type=source_type_enum,
-            github_event_type=github_event_type,
-            github_check_run_id=github_check_run_id,
-            github_installation_id=github_installation_id,
-            github_base_repo_url=github_base_repo_url,
             origin=origin,
         )
 
@@ -1365,18 +1358,20 @@ class OpenShift:
             "THOTH_ADVISER_DEV": "1" if dev else "0",
             "THOTH_LOG_ADVISER": "DEBUG" if debug else "INFO",
             "THOTH_ADVISER_RECOMMENDATION_TYPE": recommendation_type,
+            "THOTH_AUTHENTICATED_ADVISE": "1" if authenticated else "0",
             "THOTH_ADVISER_PREDICTOR_CONFIG": (
                 json.dumps(predictor_config) if predictor_config else "{}"
             ),
             "THOTH_ADVISER_METADATA": json.dumps(
                 {
-                    "github_event_type": github_event_type,
-                    "github_check_run_id": github_check_run_id,
-                    "github_installation_id": github_installation_id,
-                    "github_base_repo_url": github_base_repo_url,
                     "origin": origin,
                     "re_run_adviser_id": re_run_adviser_id,
                     "source_type": source_type if source_type is not None else None,
+                    "kebechet_metadata": kebechet_metadata,
+                    "thoth.adviser": {
+                        "justification": justification,
+                        "stack_info": stack_info,
+                    },
                 }
             ),
         }
@@ -1387,6 +1382,19 @@ class OpenShift:
         if count is not None:
             template_parameters["THOTH_ADVISER_COUNT"] = str(count)
 
+        if (
+            kebechet_metadata
+            and kebechet_metadata.get("message_justification")
+            == InternalTriggerEnum.NEW_RELEASE.value
+        ):
+            template_parameters["THOTH_ADVISER_DEPLOYMENT_PACKAGE_UPDATE"] = json.dumps(
+                {
+                    "package_name": kebechet_metadata.get("package_name"),
+                    "package_version": kebechet_metadata.get("package_version"),
+                    "index_url": kebechet_metadata.get("package_index"),
+                }
+            )
+
         workflow_parameters = self._assign_workflow_parameters_for_ceph()
 
         return self._schedule_workflow(
@@ -1415,7 +1423,11 @@ class OpenShift:
         origin: Optional[str] = None,
         whitelisted_sources: Optional[List[str]] = None,
         debug: bool = False,
+        authenticated: bool = False,
         job_id: Optional[str] = None,
+        kebechet_metadata: Optional[Dict[str, Any]] = None,
+        justification: Optional[List[Dict[str, Any]]] = None,
+        stack_info: Optional[List[Dict[str, Any]]] = None,
     ) -> Optional[str]:
         """Run provenance checks on the provided user input."""
         if not self.backend_namespace:
@@ -1426,7 +1438,17 @@ class OpenShift:
         job_id = job_id or self.generate_id("provenance-checker")
 
         template_parameters = {
-            "THOTH_ADVISER_METADATA": json.dumps({"origin": origin}),
+            "THOTH_ADVISER_METADATA": json.dumps(
+                {
+                    "origin": origin,
+                    "kebechet_metadata": kebechet_metadata,
+                    "thoth.adviser": {
+                        "justification": justification,
+                        "stack_info": stack_info,
+                    },
+                }
+            ),
+            "THOTH_AUTHENTICATED_PROVENANCE_CHECK": "1" if authenticated else "0",
             "THOTH_WHITELISTED_SOURCES": ",".join(whitelisted_sources or []),
             "THOTH_LOG_ADVISER": "DEBUG" if debug else "INFO",
             "THOTH_PROVENANCE_CHECKER_JOB_ID": job_id,
@@ -1443,47 +1465,13 @@ class OpenShift:
             },
         )
 
-    def schedule_qebhwt_workflow(
-        self,
-        github_event_type: str,
-        github_check_run_id: int,
-        github_installation_id: int,
-        github_base_repo_url: str,
-        github_head_repo_url: str,
-        origin: str,
-        revision: str,
-        host: str,
-        *,
-        job_id: Optional[str] = None,
-    ) -> Optional[str]:
-        """Schedule Workflow for Qeb-Hwt GitHub App.."""
-        workflow_id = job_id or self.generate_id("qeb-hwt")
-        template_parameters = {
-            "WORKFLOW_ID": workflow_id,
-            "GITHUB_EVENT_TYPE": github_event_type,
-            "GITHUB_CHECK_RUN_ID": str(github_check_run_id),
-            "GITHUB_INSTALLATION_ID": str(github_installation_id),
-            "GITHUB_BASE_REPO_URL": github_base_repo_url,
-            "GITHUB_HEAD_REPO_URL": github_head_repo_url,
-            "ORIGIN": origin,
-            "REVISION": revision,
-            "THOTH_HOST": host,
-        }
-
-        workflow_parameters = self._assign_workflow_parameters_for_ceph()
-
-        return self._schedule_workflow(
-            workflow=self.workflow_manager.submit_qebhwt,
-            parameters={
-                "template_parameters": template_parameters,
-                "workflow_parameters": workflow_parameters,
-            },
-        )
-
     def schedule_mi_workflow(
         self,
-        repository: str,
+        repository: Optional[str] = None,
         entities: Optional[str] = None,
+        knowledge_path: Optional[str] = None,
+        mi_used_for_thoth: Optional[bool] = False,
+        mi_merge: Optional[bool] = False,
         *,
         job_id: Optional[str] = None,
     ) -> Optional[str]:
@@ -1498,6 +1486,9 @@ class OpenShift:
             "WORKFLOW_ID": workflow_id,
             "REPOSITORY": repository,
             "ENTITIES": entities,
+            "KNOWLEDGE_PATH": knowledge_path,
+            "MI_THOTH": "1" if mi_used_for_thoth else "0",
+            "MI_MERGE": "1" if mi_merge else "0",
         }
 
         return self._schedule_workflow(
@@ -1516,9 +1507,15 @@ class OpenShift:
     ) -> Optional[str]:
         """Schedule Kebechet Workflow for a Webhook from GitHub App.."""
         workflow_id = job_id or self.generate_id("kebechet-job")
+        try:
+            slug = webhook_payload["repository"]["full_name"]
+        except KeyError:
+            slug = None
+
         template_parameters = {
             "WORKFLOW_ID": workflow_id,
             "WEBHOOK_PAYLOAD": json.dumps(webhook_payload),
+            "KEBECHET_SLUG": slug,
         }
 
         return self._schedule_workflow(
@@ -1562,6 +1559,7 @@ class OpenShift:
         repo_url: str,
         service_name: str = "github",
         *,
+        kebechet_metadata: Optional[Dict[str, Any]] = None,
         job_id: Optional[str] = None,
     ) -> Optional[str]:
         """Schedule Kebechet Workflow for a particular valid slug and service name."""
@@ -1570,6 +1568,7 @@ class OpenShift:
             "WORKFLOW_ID": workflow_id,
             "KEBECHET_REPO_URL": repo_url,
             "KEBECHET_SERVICE_NAME": service_name,
+            "KEBECHET_METADATA": json.dumps(kebechet_metadata),
         }
 
         return self._schedule_workflow(
@@ -1868,7 +1867,7 @@ class OpenShift:
                     response.to_dict(),
                 )
             except OpenShiftNotFoundError as exc:
-                raise NotFoundException(
+                raise NotFoundExceptionError(
                     f"The given Workflow {name} could not be found"
                 ) from exc
 
@@ -1890,7 +1889,7 @@ class OpenShift:
                     response.to_dict(),
                 )
             except OpenShiftNotFoundError as exc:
-                raise NotFoundException(
+                raise NotFoundExceptionError(
                     f"The given Workflow containing label {label_selector} could not be found"
                 ) from exc
 
@@ -1922,8 +1921,8 @@ class OpenShift:
                 "started_at": wf_status.get("startedAt"),
                 "state": wf_status.get("phase", "pending").lower(),
             }
-        except NotFoundException as exc:
-            raise NotFoundException(
+        except NotFoundExceptionError as exc:
+            raise NotFoundExceptionError(
                 f"Requested status for analysis {workflow_id!r} was not found"
             ) from exc
         return status
@@ -1940,3 +1939,89 @@ class OpenShift:
         )
         to_ret: Dict[str, Any] = wf["status"]
         return to_ret
+
+    def schedule_purge_solver_job(
+        self,
+        *,
+        os_name: str,
+        os_version: str,
+        python_version: str,
+        debug: bool = False,
+        job_id: Optional[str] = None,
+    ) -> Optional[str]:
+        """Schedule a job for purging solver data."""
+        workflow_id = job_id or self.generate_id("purge-solver")
+        template_parameters = {
+            "THOTH_PURGE_JOB_SUBCOMMAND": "solver",
+            "THOTH_PURGE_WORKFLOW_ID": workflow_id,
+            "THOTH_PURGE_OPERATING_SYSTEM_NAME": os_name,
+            "THOTH_PURGE_OPERATING_SYSTEM_VERSION": os_version,
+            "THOTH_PURGE_PYTHON_VERSION": python_version,
+            "THOTH_PURGE_DEBUG": str(int(debug)),
+        }
+
+        return self._schedule_workflow(
+            workflow=self.workflow_manager.submit_purge,
+            parameters={
+                "template_parameters": template_parameters,
+                "workflow_parameters": {},
+            },
+        )
+
+    def schedule_purge_adviser_job(
+        self,
+        *,
+        end_datetime: Optional[datetime.datetime] = None,
+        adviser_version: Optional[str] = None,
+        debug: bool = False,
+        job_id: Optional[str] = None,
+    ) -> Optional[str]:
+        """Schedule a job for purging adviser data."""
+        workflow_id = job_id or self.generate_id("purge-adviser")
+        template_parameters = {
+            "THOTH_PURGE_JOB_SUBCOMMAND": "adviser",
+            "THOTH_PURGE_WORKFLOW_ID": workflow_id,
+            "THOTH_PURGE_ADVISER_END_DATETIME": format_datetime(end_datetime)
+            if end_datetime
+            else "",
+            "THOTH_PURGE_ADVISER_VERSION": adviser_version if adviser_version else "",
+            "THOTH_PURGE_DEBUG": str(int(debug)),
+        }
+
+        return self._schedule_workflow(
+            workflow=self.workflow_manager.submit_purge,
+            parameters={
+                "template_parameters": template_parameters,
+                "workflow_parameters": {},
+            },
+        )
+
+    def schedule_purge_package_extract_job(
+        self,
+        *,
+        end_datetime: Optional[datetime.datetime] = None,
+        package_extract_version: Optional[str] = None,
+        debug: bool = False,
+        job_id: Optional[str] = None,
+    ) -> Optional[str]:
+        """Schedule a job for purging package-extract data."""
+        workflow_id = job_id or self.generate_id("purge-package-extract")
+        template_parameters = {
+            "THOTH_PURGE_JOB_SUBCOMMAND": "package-extract",
+            "THOTH_PURGE_WORKFLOW_ID": workflow_id,
+            "THOTH_PURGE_PACKAGE_EXTRACT_END_DATETIME": format_datetime(end_datetime)
+            if end_datetime
+            else "",
+            "THOTH_PURGE_PACKAGE_EXTRACT_VERSION": package_extract_version
+            if package_extract_version
+            else "",
+            "THOTH_PURGE_DEBUG": str(int(debug)),
+        }
+
+        return self._schedule_workflow(
+            workflow=self.workflow_manager.submit_purge,
+            parameters={
+                "template_parameters": template_parameters,
+                "workflow_parameters": {},
+            },
+        )
