@@ -567,6 +567,31 @@ class WorkflowManager:
 
         return workflow_id
 
+    def submit_purge(
+        self,
+        template_parameters: Optional[Dict[str, str]] = None,
+        workflow_parameters: Optional[Dict[str, Any]] = None,
+    ) -> Optional[str]:
+        """Submit purge workflow."""
+        if not self.openshift.infra_namespace:
+            raise ConfigurationError("Infra namespace was not provided.")
+
+        if not self.openshift.middletier_namespace:
+            raise ConfigurationError("Middletier namespace was not provided.")
+
+        template_parameters = template_parameters or {}
+        workflow_parameters = workflow_parameters or {}
+
+        workflow_id: Optional[str] = self.submit_workflow_from_template(
+            self.openshift.middletier_namespace,
+            label_selector="template=purge",
+            template_parameters=template_parameters,
+            workflow_parameters=workflow_parameters,
+            workflow_namespace=self.openshift.backend_namespace,
+        )
+
+        return workflow_id
+
     def submit_package_extract(
         self,
         template_parameters: Optional[Dict[str, str]] = None,
@@ -761,31 +786,6 @@ class WorkflowManager:
 
         return workflow_id
 
-    def submit_qebhwt(
-        self,
-        template_parameters: Optional[Dict[str, str]] = None,
-        workflow_parameters: Optional[Dict[str, Any]] = None,
-    ) -> Optional[str]:
-        """Submit Workflow for Qeb-Hwt GitHub App."""
-        if not self.openshift.infra_namespace:
-            raise ConfigurationError("Infra namespace was not provided.")
-
-        if not self.openshift.backend_namespace:
-            raise ConfigurationError("Backend namespace was not provided.")
-
-        template_parameters = template_parameters or {}
-        workflow_parameters = workflow_parameters or {}
-
-        workflow_id: Optional[str] = self.submit_workflow_from_template(
-            self.openshift.infra_namespace,
-            label_selector="template=qeb-hwt",
-            template_parameters=template_parameters,
-            workflow_parameters=workflow_parameters,
-            workflow_namespace=self.openshift.backend_namespace,
-        )
-
-        return workflow_id
-
     def submit_security_indicator(
         self,
         template_parameters: Optional[Dict[str, str]] = None,
